@@ -4,16 +4,13 @@ from typing import Optional
 
 from configargparse import ArgumentParser, ArgumentTypeError, Namespace
 
-from pyscreener import _version
+from pyscreener.utils.utils import Reduction
 
-TIMESTAMP = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
+__version__ = "1.2.1"
 
-__version__ = _version.get_versions()['version']
 
 def gen_args(argv: Optional[str] = None) -> Namespace:
-    parser = ArgumentParser(
-        description="Automate virtual screening of compound libraries."
-    )
+    parser = ArgumentParser(description="Automate virtual screening of compound libraries.")
 
     add_general_args(parser)
     add_preprocessing_args(parser)
@@ -84,14 +81,9 @@ def add_preprocessing_args(parser: ArgumentParser):
 
 
 def add_supply_args(parser: ArgumentParser):
+    parser.add_argument("-s", "--smis", nargs="+", help="the SMILES strings of the ligands to dock")
     parser.add_argument(
-        "-s", "--smis", nargs="+", help="the SMILES strings of the ligands to dock"
-    )
-    parser.add_argument(
-        "-i",
-        "--input-files",
-        nargs="+",
-        help="the filenames containing ligands to dock",
+        "-i", "--input-files", nargs="+", help="the filenames containing ligands to dock"
     )
     parser.add_argument(
         "--input-filetypes",
@@ -139,11 +131,7 @@ def add_screen_args(parser: ArgumentParser):
         required=True,
         help="the type of docking screen to perform",
     )
-    parser.add_argument(
-        "--receptors",
-        nargs="+",
-        help="the filenames of the receptors",
-    )
+    parser.add_argument("--receptors", nargs="+", help="the filenames of the receptors")
     parser.add_argument(
         "--center",
         type=float,
@@ -160,9 +148,7 @@ def add_screen_args(parser: ArgumentParser):
     )
     parser.add_argument("--metadata-template", type=json.loads, default={})
     parser.add_argument(
-        "--pdbids",
-        nargs="+",
-        help="the PDB IDs of the crystal structures to dock against",
+        "--pdbids", nargs="+", help="the PDB IDs of the crystal structures to dock against"
     )
     parser.add_argument(
         "--docked-ligand-file",
@@ -177,28 +163,19 @@ def add_screen_args(parser: ArgumentParser):
     parser.add_argument("-nc", "--ncpu", default=1, type=int)
     parser.add_argument("--base-name", default="ligand")
     parser.add_argument(
-        "--score-mode",
+        "--reduction",
         default="best",
-        choices=("best", "avg", "boltzmann", "top-k"),
+        choices=set(r.name for r in Reduction),
         help="The method used to calculate the score of a single docking run on a single receptor",
     )
     parser.add_argument(
-        "--repeat-score-mode",
+        "--receptor-reduction",
         default="best",
-        choices=("best", "avg", "boltzmann", "top-k"),
-        help="The method used to calculate the overall score from repeated docking runs",
-    )
-    parser.add_argument(
-        "--ensemble-score-mode",
-        default="best",
-        choices=("best", "avg", "boltzmann", "top-k"),
+        choices=set(r.name for r in Reduction),
         help="The method used to calculate the overall score from an ensemble of docking runs",
     )
     parser.add_argument(
-        "--repeats",
-        default=1,
-        type=int,
-        help="the number of times to repeat each docking run",
+        "--repeats", default=1, type=int, help="the number of times to repeat each docking run"
     )
     parser.add_argument(
         "-k",
